#define JPEG_INTERNALS
#include "jinclude.h"
#include "jpeglib.h"
#include "jdrest.h"

struct my_rest_struct
{
  struct jpeg_restoration pub;
};

typedef struct my_rest_struct * my_rest_ptr;

/* Prototypes - not in IJG style. */
METHODDEF(void) dummy_decompress_plane
  JPP((j_decompress_ptr cinfo, struct jpeg_restoration * rest,
    JSAMPARRAY output_buf, JBLOCKARRAY blockarray));

GLOBAL(void)
jpeg_rest_dummy(j_decompress_ptr cinfo, J_REST_TARGET target)
{
  my_rest_ptr rest;

  if (target != JREST_TARGET_IDCT_Y &&
    target != JREST_TARGET_IDCT_Cb &&
    target != JREST_TARGET_IDCT_Cr)
  {
    ERREXIT(cinfo, JERR_REST_NOT_APPLICABLE);
  }

  /* The first restoration */
  rest = (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_IMAGE,
				SIZEOF(struct my_rest_struct));

  rest->pub.decompress_plane = dummy_decompress_plane;
  rest->pub.target = target;
  add_restorator(cinfo, (struct jpeg_restoration *) rest);
}

METHODDEF(void)
dummy_decompress_plane(j_decompress_ptr cinfo, struct jpeg_restoration * rest,
    JSAMPARRAY output_ptr, JBLOCKARRAY blockarray)
{
  JDIMENSION last_iMCU_row = cinfo->total_iMCU_rows - 1;
  JDIMENSION block_num;
  int block_row, block_rows;
  JDIMENSION output_col;
  jpeg_component_info *compptr;
  inverse_DCT_method_ptr inverse_DCT;
  int i,j;


  compptr = &cinfo->comp_info[rest->component];

  /* Align the virtual buffer for this component. */
  /* Count non-dummy DCT block rows in this iMCU row. */
  if (cinfo->output_iMCU_row < last_iMCU_row)
    block_rows = compptr->v_samp_factor;
  else {
    /* NB: can't use last_row_height here; it is input-side-dependent! */
    block_rows = (int) (compptr->height_in_blocks % compptr->v_samp_factor);
    if (block_rows == 0) block_rows = compptr->v_samp_factor;
  }
  inverse_DCT = cinfo->idct->inverse_DCT[rest->component];
  /* Loop over all DCT blocks to be processed. */
  for (block_row = 0; block_row < block_rows; block_row++) {
    /*buffer_ptr = buffer[block_row];*/
    output_col = 0;
    for (block_num = 0; block_num < compptr->width_in_blocks; block_num++) {
      /*(*inverse_DCT) (cinfo, compptr, (JCOEFPTR) buffer_ptr,
		      output_ptr, output_col);*/
      for(i=0; i<8;i++)
	for(j=0; j<8;j++)
	{
	  /* i: row
	   * j: column */
	  /*
	  (output_ptr[i] + output_col)[j] =
	    blockarray[cinfo->output_iMCU_row * compptr->v_samp_factor
	      + block_row][block_num][i*8+j];
	      */
	  (output_ptr[i] + output_col)[j] = 50;
	}
      /*buffer_ptr++;*/
      output_col += compptr->DCT_scaled_size;
    }
    output_ptr += compptr->DCT_scaled_size;
  }
}
